﻿using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using System.IO;
using System.Reflection;
using Xamarin.Forms;

namespace GettingStarted
{
    #region Sample Page
    /// <summary>
    /// Provides the implementation for GettingStartedDocIO class.
    /// </summary>
    public partial class GettingStartedDocIO : ContentPage
    {
        #region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="GettingStartedDocIO"/> class.
        /// </summary>
        public GettingStartedDocIO()
        {
            // Assign page instance to Command.
            GettingStartedDocIOCommand.gettingStartedDocIO = this;
            InitializeComponent();
            if (Device.Idiom != TargetIdiom.Phone && Device.OS == TargetPlatform.Windows)
            {
                
                Content_1.HorizontalOptions = LayoutOptions.Center;
                
                btnGenerate.HorizontalOptions = LayoutOptions.Center;

                
                Content_1.VerticalOptions = LayoutOptions.Center;
                
                btnGenerate.VerticalOptions = LayoutOptions.Center;
                btnGenerate.BackgroundColor = Color.Gray;
            }
            else if (Device.Idiom == TargetIdiom.Phone && Device.OS == TargetPlatform.Windows)
            {
                if (!App.isUWP)
                {
                    Content_1.FontSize = 18.5;
                }
                else
                {
                    Content_1.FontSize = 13.5;
                }
                
                Content_1.VerticalOptions = LayoutOptions.Center;
                btnGenerate.VerticalOptions = LayoutOptions.Center;
            }
            
        }
        #endregion
       
    }
    #endregion
    #region Command Implementation
    /// <summary>
    /// Provides the implementation for GettingStartedDocIOCommand class.
    /// </summary>
    public class GettingStartedDocIOCommand : CommandBase
    {
        #region Fields
        public static GettingStartedDocIO gettingStartedDocIO;
        public string content;
        #endregion
        #region Constructor
        public GettingStartedDocIOCommand()
        {
        }
        #endregion

        #region Implementation
        /// <summary>
        /// Defines the method to be called when the command is invoked.
        /// </summary>
        /// <param name="parameter">
        /// Data used by the command. If the command does not require data to be passed,
        /// this object can be set to null.
        /// </param>
        protected override void ExecuteCommand(object parameter)
        {
            GetContentFromWordDocument();
        }
        private void GetContentFromWordDocument()
        {
           
            // Gets assembly.
            Assembly assembly = typeof(App).GetTypeInfo().Assembly;
            // Gets input Word document and Word document to clone from embedded resource collection.
            Stream inputStream = assembly.GetManifestResourceStream("GettingStarted.Templates.Sample.docx");
            // Loads existing Word document using Essential DocIO.
            using (WordDocument document = new WordDocument(inputStream))
            {
                //Iterates sections from document.           
                foreach (WSection section in document.Sections)
                {
                    //Iterates textbody from section.
                    foreach (WTextBody textBody in section.ChildEntities)
                    {
                        //Iterates textbody items.
                        for (int index = 0; index < textBody.ChildEntities.Count; index++)
                        {
                            Entity entity = textBody.ChildEntities[index];
                            //Split paragraph based on format.
                            SplitParagraph(entity);
                        }

                    }
                }
                MemoryStream stream = new MemoryStream();
                document.Save(stream, FormatType.Docx);
                document.Close();

                if (Device.OS == TargetPlatform.WinPhone || Device.OS == TargetPlatform.Windows)
                    DependencyService.Get<ISaveWindowsPhone>()
                        .Save("GeneratedSample.docx", "application/msword", stream);
                else
                    DependencyService.Get<ISave>().Save("GeneratedSample.docx", "application/msword", stream);
            }          
              
        }
        /// <summary>
        /// Splits paragraph based on the character formats of para items.
        /// </summary>
        /// <param name="entity"></param>
        public void SplitParagraph(Entity entity)
        {
            switch (entity.EntityType)
            {
                case EntityType.Paragraph:
                    WParagraph paragraph = entity as WParagraph;

                    WCharacterFormat previousCharacterFormat = null;
                    if (paragraph != null)
                    {
                        //Iterates paragraph items.
                        for (int paraItemIndex = 0; paraItemIndex < paragraph.Items.Count; paraItemIndex++)
                        {
                            Entity paraItem = paragraph.Items[paraItemIndex];

                            if (paraItem is WTextRange)
                            {
                                //Get the current text range item.
                                WTextRange textRange = paragraph.Items[paraItemIndex] as WTextRange;
                                //Gets character format of current text range.
                                WCharacterFormat currentCharacterFormat = textRange.CharacterFormat;

                                //Compares character format with previous textrange character format.
                                if (previousCharacterFormat != null && (previousCharacterFormat.Italic != currentCharacterFormat.Italic))
                                {
                                    //Gets index of paragraph in textbody.
                                    int paraIndex = paragraph.OwnerTextBody.ChildEntities.IndexOf(paragraph);
                                    //New paragraph to move items.                              
                                    WParagraph splittedParagraph = new WParagraph(paragraph.Document);
                                    //Inserts new paragraph in the textbody.
                                    paragraph.OwnerTextBody.ChildEntities.Insert(paraIndex + 1, splittedParagraph);

                                    //CLone items from one paragraph and add into another paragraph.
                                    for (int index = paraItemIndex; paragraph.Items.Count > index; index++)
                                    {
                                        Entity clonedItem = paragraph.Items[index].Clone();
                                        splittedParagraph.ChildEntities.Add(clonedItem);
                                    }
                                    //Removes items from old paragraph.
                                    for (; paragraph.Items.Count > paraItemIndex; paraItemIndex++)
                                    {
                                        paragraph.Items.Remove(paragraph.Items[paraItemIndex]);
                                    }

                                    //Sets paragraph formats for splitted paragraph.
                                    splittedParagraph.ParagraphFormat.BeforeSpacing = 15f;
                                }

                                //Maintains current textrange character format and read next item.
                                previousCharacterFormat = textRange.CharacterFormat;
                            }

                        }
                    }
                    break;
            }
        }
        #endregion
    }
    #endregion
}